@@ -27,12 +27,10 @@
 
 #include <gtk/gtk.h>
 
-#ifdef BUILD_GNOME2
-
-#include <libgnome/gnome-help.h>
-#include <libgnomeui/libgnomeui.h>
-
-#endif
+#include <libxml/xmlerror.h>
+#include <libxml/xmlwriter.h>
+#include <libxml/parser.h>
+#include <libxml/xmlstring.h>
 
 #include "brasero-app.h"
 #include "brasero-utils.h"
@@ -43,6 +41,9 @@
 #include "brasero-session.h"
 #include "brasero-project-manager.h"
 #include "brasero-pref.h"
+#include "brasero-image-option-dialog.h"
+#include "brasero-disc-copy-dialog.h"
+#include "brasero-burn-dialog.h"
 #include "burn-debug.h"
 #include "burn-drive.h"
 #include "burn-caps.h"
@@ -52,6 +53,14 @@
 typedef struct _BraseroAppPrivate BraseroAppPrivate;
 struct _BraseroAppPrivate
 {
+	GdkWindow *parent;
+
+	GtkWidget *mainwin;
+
+	GtkWidget *burn_dialog;
+	GtkWidget *tool_dialog;
+
+	/* This is the toplevel window currently displayed */
 	GtkWidget *toplevel;
 
 	GtkWidget *projects;
@@ -65,14 +74,18 @@ struct _BraseroAppPrivate
 	gint width;
 	gint height;
 
+	gchar *saved_contents;
 	guint is_maximised:1;
-	guint is_running:1;
 };
 
 #define BRASERO_APP_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_APP, BraseroAppPrivate))
 
 
-G_DEFINE_TYPE (BraseroApp, brasero_app, GTK_TYPE_WINDOW);
+G_DEFINE_TYPE (BraseroApp, brasero_app, G_TYPE_OBJECT);
+
+
+#define SESSION_VERSION "0.1"
+#define BRASERO_SESSION_TMP_SESSION_PATH	"brasero.session"
 
 /**
  * Menus and toolbar
@@ -86,10 +99,7 @@ static void on_integrity_check_cb (GtkAction *action, BraseroApp *app);
 static void on_exit_cb (GtkAction *action, BraseroApp *app);
 
 static void on_about_cb (GtkAction *action, BraseroApp *app);
-
-#ifdef BUILD_GNOME2
 static void on_help_cb (GtkAction *action, BraseroApp *app);
-#endif
 
 static GtkActionEntry entries[] = {
 	{"ProjectMenu", NULL, N_("_Project")},
@@ -114,13 +124,9 @@ static GtkActionEntry entries[] = {
 	{"Exit", GTK_STOCK_QUIT, NULL, NULL,
 	 N_("Exit the program"), G_CALLBACK (on_exit_cb)},
 
-#ifdef BUILD_GNOME2
-	
 	{"Contents", GTK_STOCK_HELP, N_("_Contents"), "F1", N_("Display help"),
 	 G_CALLBACK (on_help_cb)}, 
 
-#endif
-
 	{"About", GTK_STOCK_ABOUT, NULL, NULL, N_("About"),
 	 G_CALLBACK (on_about_cb)},
 };
@@ -149,17 +155,297 @@ static const gchar *description = {
 		"<menuitem action='Check'/>"
 	    "</menu>"
 	    "<menu action='HelpMenu'>"
-
-#ifdef BUILD_GNOME2
 		"<menuitem action='Contents'/>"
 		"<separator/>"
-#endif
 		"<menuitem action='About'/>"
 	    "</menu>"
 	    "</menubar>"
 	"</ui>"
 };
 
+static gchar *
+brasero_app_get_path (const gchar *name)
+{
+	gchar *directory;
+	gchar *retval;
+
+	directory = g_build_filename (g_get_user_config_dir (),
+				      "brasero",
+				      NULL);
+	if (!g_file_test (directory, G_FILE_TEST_EXISTS))
+		g_mkdir_with_parents (directory, S_IRWXU);
+
+	retval = g_build_filename (directory, name, NULL);
+	g_free (directory);
+	return retval;
+}
+
+static gboolean
+brasero_app_load_window_state (BraseroApp *app)
+{
+	gchar *height_str = NULL;
+	gchar *width_str = NULL;
+	gchar *state_str = NULL;
+	gchar *version = NULL;
+	gint height;
+	gint width;
+	gint state = 0;
+
+	gchar *session_path;
+	xmlNodePtr item;
+	xmlDocPtr session = NULL;
+
+	GdkScreen *screen;
+	GdkRectangle rect;
+	gint monitor;
+
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+
+	/* Make sure that on first run the window has a default size of at least
+	 * 85% of the screen (hardware not GTK+) */
+	screen = gtk_window_get_screen (GTK_WINDOW (priv->mainwin));
+	monitor = gdk_screen_get_monitor_at_window (screen, GTK_WIDGET (priv->mainwin)->window);
+	gdk_screen_get_monitor_geometry (screen, monitor, &rect);
+	width = rect.width / 100 * 85;
+	height = rect.height / 100 * 85;
+
+	session_path = brasero_app_get_path (BRASERO_SESSION_TMP_SESSION_PATH);
+	if (!session_path)
+		goto end;
+
+	session = xmlParseFile (session_path);
+	g_free (session_path);
+
+	if (!session)
+		goto end;
+
+	item = xmlDocGetRootElement (session);
+	if (!item)
+		goto end;
+
+	if (xmlStrcmp (item->name, (const xmlChar *) "Session") || item->next)
+		goto end;
+
+	item = item->children;
+	while (item) {
+		if (!xmlStrcmp (item->name, (const xmlChar *) "version")) {
+			if (version)
+				goto end;
+
+			version = (char *) xmlNodeListGetString (session,
+								 item->xmlChildrenNode,
+								 1);
+		}
+		else if (!xmlStrcmp (item->name, (const xmlChar *) "width")) {
+			if (width_str)
+				goto end;
+
+			width_str = (char *) xmlNodeListGetString (session,
+								   item->xmlChildrenNode,
+								   1);
+		}
+		else if (!xmlStrcmp (item->name, (const xmlChar *) "height")) {
+			if (height_str)
+				goto end;
+
+			height_str = (char *) xmlNodeListGetString (session,
+								    item->xmlChildrenNode,
+								    1);
+		}
+		else if (!xmlStrcmp (item->name, (const xmlChar *) "state")) {
+			if (state_str)
+				goto end;
+
+			state_str = (char *) xmlNodeListGetString (session,
+								   item->xmlChildrenNode,
+								   1);
+		}
+		else if (item->type == XML_ELEMENT_NODE)
+			goto end;
+
+		item = item->next;
+	}
+
+	if (!version || strcmp (version, SESSION_VERSION))
+		goto end;
+
+	/* restore the window state */
+	if (height_str)
+		height = (int) g_strtod (height_str, NULL);
+
+	if (width_str)
+		width = (int) g_strtod (width_str, NULL);
+
+	if (state_str)
+		state = (int) g_strtod (state_str, NULL);
+
+end:
+	if (height_str)
+		g_free (height_str);
+
+	if (width_str)
+		g_free (width_str);
+
+	if (state_str)
+		g_free (state_str);
+
+	if (version)
+		g_free (version);
+
+	xmlFreeDoc (session);
+
+	if (width && height)
+		gtk_window_resize (GTK_WINDOW (priv->mainwin),
+				   width,
+				   height);
+
+	if (state)
+		gtk_window_maximize (GTK_WINDOW (priv->mainwin));
+
+	return TRUE;
+}
+
+void
+brasero_app_save_window_state (BraseroApp *app)
+{
+	gint success;
+	gchar *session_path;
+	xmlTextWriter *session;
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+
+	/* now save the state of the window */
+	session_path = brasero_app_get_path (BRASERO_SESSION_TMP_SESSION_PATH);
+	if (!session_path)
+		return;
+
+	/* write information */
+	session = xmlNewTextWriterFilename (session_path, 0);
+	if (!session) {
+		g_free (session_path);
+		return;
+	}
+
+	xmlTextWriterSetIndent (session, 1);
+	xmlTextWriterSetIndentString (session, (xmlChar *) "\t");
+
+	success = xmlTextWriterStartDocument (session,
+					      NULL,
+					      NULL,
+					      NULL);
+	if (success < 0)
+		goto error;
+
+	success = xmlTextWriterStartElement (session,
+					     (xmlChar *) "Session");
+	if (success < 0)
+		goto error;
+
+	success = xmlTextWriterWriteElement (session,
+					     (xmlChar *) "version",
+					     (xmlChar *) SESSION_VERSION);
+	if (success < 0)
+		goto error;
+
+	success = xmlTextWriterWriteFormatElement (session,
+						   (xmlChar *) "width",
+						   "%i",
+						   priv->width);
+	if (success < 0)
+		goto error;
+
+	success = xmlTextWriterWriteFormatElement (session,
+						   (xmlChar *) "height",
+						   "%i",
+						   priv->height);
+	if (success < 0)
+		goto error;
+
+	success = xmlTextWriterWriteFormatElement (session,
+						   (xmlChar *) "state",
+						   "%i",
+						   priv->is_maximised);
+	if (success < 0)
+		goto error;
+
+	success = xmlTextWriterEndElement (session);
+	if (success < 0)
+		goto error;
+
+	xmlTextWriterEndDocument (session);
+	xmlFreeTextWriter (session);
+	g_free (session_path);
+	return;
+
+error:
+	xmlTextWriterEndDocument (session);
+	xmlFreeTextWriter (session);
+	g_remove (session_path);
+	g_free (session_path);
+}
+
+/**
+ * returns FALSE when nothing prevents the shutdown
+ * returns TRUE when shutdown should be delayed
+ */
+
+gboolean
+brasero_app_save_contents (BraseroApp *app,
+			   gboolean cancellable)
+{
+	gboolean cancel;
+	gchar *project_path;
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+
+	if (priv->burn_dialog) {
+		if (cancellable)
+			return (brasero_burn_dialog_cancel (BRASERO_BURN_DIALOG (priv->burn_dialog)) == FALSE);
+
+		gtk_widget_destroy (priv->burn_dialog);
+		return FALSE;
+	}
+
+	if (priv->tool_dialog) {
+		if (cancellable)
+			return (brasero_tool_dialog_cancel (BRASERO_TOOL_DIALOG (priv->tool_dialog)) == FALSE);
+
+		gtk_widget_destroy (priv->tool_dialog);
+		return FALSE;
+	}
+
+	/* If we are not having a main window there is no point in going further */
+	if (!priv->mainwin)
+		return FALSE;
+
+	if (priv->saved_contents) {
+		g_free (priv->saved_contents);
+		priv->saved_contents = NULL;
+	}
+
+	project_path = brasero_app_get_path (BRASERO_SESSION_TMP_PROJECT_PATH);
+	cancel = brasero_project_manager_save_session (BRASERO_PROJECT_MANAGER (priv->projects),
+						       project_path,
+						       &priv->saved_contents,
+						       cancellable);
+	g_free (project_path);
+
+	return cancel;
+}
+
+const gchar *
+brasero_app_get_saved_contents (BraseroApp *app)
+{
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+	return priv->saved_contents;
+}
+
 /**
  * These functions are only useful because they set the proper toplevel parent
  * for the message dialog. The following one also sets some properties in case
@@ -189,8 +475,8 @@ brasero_app_dialog (BraseroApp *app,
 
 	priv = BRASERO_APP_PRIVATE (app);
 
-	if (priv->is_running)
-		toplevel = GTK_WINDOW (app);
+	if (priv->mainwin)
+		toplevel = GTK_WINDOW (priv->mainwin);
 	else if (!priv->toplevel) {
 		is_on_top = TRUE;
 		toplevel = NULL;
@@ -205,6 +491,12 @@ brasero_app_dialog (BraseroApp *app,
 					 button_type,
 					 primary_message);
 
+	if (!toplevel && priv->parent) {
+		gtk_widget_realize (GTK_WIDGET (dialog));
+		gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+		gdk_window_set_transient_for (GTK_WIDGET (dialog)->window, priv->parent);
+	}
+
 	if (is_on_top) {
 		gtk_window_set_skip_pager_hint (GTK_WINDOW (dialog), FALSE);
 		gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
@@ -226,24 +518,37 @@ brasero_app_alert (BraseroApp *app,
 		   GtkMessageType type)
 {
 	GtkWidget *parent = NULL;
+	gboolean is_on_top= TRUE;
 	BraseroAppPrivate *priv;
 	GtkWidget *alert;
 
 	priv = BRASERO_APP_PRIVATE (app);
 
 	/* Whatever happens, they need a parent or must be in the taskbar */
-	if (priv->is_running)
-		parent = GTK_WIDGET (app);
-	else if (priv->toplevel)
+	if (priv->mainwin) {
+		parent = GTK_WIDGET (priv->mainwin);
+		is_on_top = FALSE;
+	}
+	else if (priv->toplevel) {
 		parent = priv->toplevel;
+		is_on_top = FALSE;
+	}
 
 	alert = brasero_utils_create_message_dialog (parent,
 						     primary_message,
 						     secondary_message,
 						     type);
-	gtk_window_set_title (GTK_WINDOW (alert), _("Disc Burner"));
 
-	if (!parent) {
+	if (!parent && priv->parent) {
+		is_on_top = FALSE;
+
+		gtk_widget_realize (GTK_WIDGET (alert));
+		gtk_window_set_modal (GTK_WINDOW (alert), TRUE);
+		gdk_window_set_transient_for (GTK_WIDGET (alert)->window, priv->parent);
+	}
+
+	if (is_on_top) {
+		gtk_window_set_title (GTK_WINDOW (alert), _("Disc Burner"));
 		gtk_window_set_skip_pager_hint (GTK_WINDOW (alert), FALSE);
 		gtk_window_set_skip_taskbar_hint (GTK_WINDOW (alert), FALSE);
 	}
@@ -290,19 +595,16 @@ brasero_app_get_project_manager (BraseroApp *app)
 	return priv->projects;
 }
 
-void
-brasero_app_get_geometry (BraseroApp *app,
-			  gint *width,
-			  gint *height,
-			  gboolean *maximised)
+static gboolean
+on_destroy_cb (GtkWidget *window, BraseroApp *app)
 {
 	BraseroAppPrivate *priv;
 
 	priv = BRASERO_APP_PRIVATE (app);
+	if (priv->mainwin)
+		gtk_main_quit ();
 
-	*width = priv->width;
-	*height = priv->height;
-	*maximised = priv->is_maximised;
+	return FALSE;
 }
 
 static gboolean
@@ -311,31 +613,30 @@ on_delete_cb (GtkWidget *window, GdkEvent *event, BraseroApp *app)
 	BraseroAppPrivate *priv;
 
 	priv = BRASERO_APP_PRIVATE (app);
-	if (!priv->is_running)
+	if (!priv->mainwin)
 		return FALSE;
 
-	return brasero_session_save (app, TRUE, TRUE);
-}
-
-static gboolean
-on_destroy_cb (GtkWidget *window, BraseroApp *app)
-{
-	BraseroAppPrivate *priv;
-
-	priv = BRASERO_APP_PRIVATE (app);
-	if (priv->is_running)
-		gtk_main_quit ();
+	if (brasero_app_save_contents (app, TRUE))
+		return TRUE;
 
+	brasero_app_save_window_state (app);
 	return FALSE;
 }
 
 static void
 on_exit_cb (GtkAction *action, BraseroApp *app)
 {
-	if (brasero_session_save (app, TRUE, TRUE))
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+
+	if (brasero_app_save_contents (app, TRUE))
 		return;
 
-	gtk_widget_destroy (GTK_WIDGET (app));
+	if (priv->mainwin) {
+		brasero_app_save_window_state (app);
+		gtk_widget_destroy (GTK_WIDGET (priv->mainwin));
+	}
 }
 
 gboolean
@@ -344,19 +645,122 @@ brasero_app_is_running (BraseroApp *app)
 	BraseroAppPrivate *priv;
 
 	priv = BRASERO_APP_PRIVATE (app);
-	return priv->is_running;
+	return (priv->mainwin != NULL);
 }
 
 void
-brasero_app_run (BraseroApp *app)
+brasero_app_set_parent (BraseroApp *app,
+			guint parent_xid)
 {
 	BraseroAppPrivate *priv;
 
 	priv = BRASERO_APP_PRIVATE (app);
 
-	priv->is_running = TRUE;
-	gtk_widget_realize (GTK_WIDGET (app));
-	brasero_session_load (app);
+	priv->parent = gdk_window_foreign_new (parent_xid);
+}
+
+gboolean
+brasero_app_burn (BraseroApp *app,
+		  BraseroBurnSession *session)
+{
+	gboolean success;
+	GtkWidget *dialog;
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+
+	/* now setup the burn dialog */
+	dialog = brasero_burn_dialog_new ();
+	priv->burn_dialog = dialog;
+
+	brasero_app_set_toplevel (app, GTK_WINDOW (dialog));
+	success = brasero_burn_dialog_run (BRASERO_BURN_DIALOG (dialog), session);
+
+	priv->burn_dialog = NULL;
+
+	/* The destruction of the dialog will bring the main window forward */
+	gtk_widget_destroy (dialog);
+	return success;
+}
+
+void
+brasero_app_burn_image (BraseroApp *app,
+			const gchar *uri)
+{
+	BraseroBurnSession *session;
+	GtkResponseType result;
+	GtkWidget *dialog;
+
+	/* setup, show, and run options dialog */
+	dialog = brasero_image_option_dialog_new ();
+	brasero_image_option_dialog_set_image_uri (BRASERO_IMAGE_OPTION_DIALOG (dialog), uri);
+
+	brasero_app_set_toplevel (app, GTK_WINDOW (dialog));
+
+	result = gtk_dialog_run (GTK_DIALOG (dialog));
+	if (result != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (dialog);
+		return;
+	}
+
+	session = brasero_burn_options_get_session (BRASERO_BURN_OPTIONS (dialog));
+	/* The destruction of the dialog will bring the main window forward */
+	gtk_widget_destroy (dialog);
+
+	brasero_app_burn (app, session);
+	g_object_unref (session);
+}
+
+void
+brasero_app_copy_disc (BraseroApp *app,
+		       const gchar *device,
+		       const gchar *cover)
+{
+	BraseroBurnSession *session;
+	GtkResponseType result;
+	GtkWidget *dialog;
+
+	dialog = brasero_disc_copy_dialog_new ();
+	brasero_app_set_toplevel (app, GTK_WINDOW (dialog));
+
+	/* if a device is specified then get the corresponding medium */
+	if (device) {
+		BraseroDrive *drive;
+		BraseroMediumMonitor *monitor;
+
+		monitor = brasero_medium_monitor_get_default ();
+		drive = brasero_medium_monitor_get_drive (monitor, device);
+		g_object_unref (monitor);
+
+		brasero_disc_copy_dialog_set_drive (BRASERO_DISC_COPY_DIALOG (dialog), drive);
+		g_object_unref (drive);
+	}
+
+	result = gtk_dialog_run (GTK_DIALOG (dialog));
+	if (result != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (dialog);
+		return;
+	}
+
+	session = brasero_burn_options_get_session (BRASERO_BURN_OPTIONS (dialog));
+
+	/* The destruction of the dialog will bring the main window forward */
+	gtk_widget_destroy (dialog);
+
+	/* Set a cover if any. */
+	if (cover) {
+		GValue *value;
+
+		value = g_new0 (GValue, 1);
+		g_value_init (value, G_TYPE_STRING);
+		g_value_set_string (value, cover);
+		brasero_burn_session_tag_add (session,
+					      BRASERO_COVER_URI,
+					      value);
+	}
+
+	brasero_app_burn (app, session);
+	g_object_unref (session);
 }
 
 void
@@ -364,12 +768,10 @@ brasero_app_blank (BraseroApp *app,
 		   const gchar *device)
 {
 	GtkWidget *dialog;
-	GtkWidget *toplevel;
 	BraseroAppPrivate *priv;
 
 	priv = BRASERO_APP_PRIVATE (app);
 	dialog = brasero_blank_dialog_new ();
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (app));
 
 	if (device) {
 		BraseroDrive *drive;
@@ -386,20 +788,29 @@ brasero_app_blank (BraseroApp *app,
 		g_object_unref (drive);
 	}
 
-	if (!priv->is_running) {
-		gtk_widget_show (dialog);
-		gtk_dialog_run (GTK_DIALOG (dialog));
+	priv->tool_dialog = dialog;
+	if (!priv->mainwin) {
+		gtk_widget_realize (dialog);
 
-		/* brasero-tool-dialog auto destroys itself */
-		// gtk_widget_destroy (dialog);
+		if (priv->parent) {
+			gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+			gdk_window_set_transient_for (GTK_WIDGET (dialog)->window, priv->parent);
+		}
 	}
 	else {
+		GtkWidget *toplevel;
+
+		/* FIXME! This is a bad idea and needs fixing */
+		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (priv->mainwin));
+
 		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
 		gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 		gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
-		
-		gtk_widget_show (dialog);
 	}
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	priv->tool_dialog = NULL;
 }
 
 static void
@@ -413,15 +824,24 @@ on_eject_cb (GtkAction *action, BraseroApp *app)
 {
 	GtkWidget *dialog;
 	GtkWidget *toplevel;
+	BraseroAppPrivate *priv;
 
-	dialog = brasero_eject_dialog_new();
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (app));
+	priv = BRASERO_APP_PRIVATE (app);
+
+	dialog = brasero_eject_dialog_new ();
+
+	/* FIXME! This is a bad idea and needs fixing */
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (priv->mainwin));
 
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
 
-	gtk_widget_show (dialog);
+	priv->tool_dialog = dialog;
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	priv->tool_dialog = NULL;
+
+	gtk_widget_destroy (dialog);
 }
 
 void
@@ -429,13 +849,12 @@ brasero_app_check (BraseroApp *app,
 		   const gchar *device)
 {
 	GtkWidget *dialog;
-	GtkWidget *toplevel;
 	BraseroAppPrivate *priv;
 
 	priv = BRASERO_APP_PRIVATE (app);
 
 	dialog = brasero_sum_dialog_new ();
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (app));
+	priv->tool_dialog = dialog;
 
 	if (device) {
 		BraseroDrive *drive;
@@ -452,20 +871,28 @@ brasero_app_check (BraseroApp *app,
 		g_object_unref (drive);
 	}
 
-	if (!priv->is_running) {
-		gtk_widget_show (dialog);
-		gtk_dialog_run (GTK_DIALOG (dialog));
+	if (!priv->mainwin) {
+		gtk_widget_realize (dialog);
 
-		/* brasero-tool-dialog auto destroys itself */
-		// gtk_widget_destroy (dialog);
+		if (priv->parent) {
+			gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+			gdk_window_set_transient_for (GTK_WIDGET (dialog)->window, priv->parent);
+		}
 	}
 	else {
+		GtkWidget *toplevel;
+
+		/* FIXME! This is a bad idea and needs fixing */
+		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (priv->mainwin));
+
 		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
 		gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 		gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
-		
-		gtk_widget_show (dialog);
 	}
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	priv->tool_dialog = NULL;
 }
 
 static void
@@ -474,14 +901,70 @@ on_integrity_check_cb (GtkAction *action, BraseroApp *app)
 	brasero_app_check (app, NULL);
 }
 
+static void
+brasero_app_current_toplevel_destroyed (GtkWidget *widget,
+					BraseroApp *app)
+{
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+	if (priv->mainwin)
+		gtk_widget_show (GTK_WIDGET (priv->mainwin));
+}
+
+void
+brasero_app_set_toplevel (BraseroApp *app, GtkWindow *window)
+{
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+
+	if (!priv->mainwin) {
+		if (priv->parent) {
+			gtk_widget_realize (GTK_WIDGET (window));
+			gtk_window_set_modal (GTK_WINDOW (window), TRUE);
+			gdk_window_set_transient_for (GTK_WIDGET (window)->window, priv->parent);
+		}
+		else {
+			gtk_window_set_skip_taskbar_hint (GTK_WINDOW (window), FALSE);
+			gtk_window_set_skip_pager_hint (GTK_WINDOW (window), FALSE);
+			gtk_window_set_type_hint (GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_NORMAL);
+			gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
+		}
+	}
+	else {
+		gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (priv->mainwin));
+		gtk_window_set_modal (GTK_WINDOW (window), TRUE);
+
+		/* hide main dialog if it is shown */
+		gtk_widget_hide (GTK_WIDGET (priv->mainwin));
+
+		gtk_window_set_skip_taskbar_hint (GTK_WINDOW (window), FALSE);
+		gtk_window_set_skip_pager_hint (GTK_WINDOW (window), FALSE);
+		gtk_window_set_type_hint (GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_NORMAL);
+		gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
+	}
+
+	gtk_widget_show (GTK_WIDGET (window));
+	g_signal_connect (window,
+			  "destroy",
+			  G_CALLBACK (brasero_app_current_toplevel_destroyed),
+			  app);
+}
+
 static void
 on_prefs_cb (GtkAction *action, BraseroApp *app)
 {
 	GtkWidget *dialog;
 	GtkWidget *toplevel;
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
 
 	dialog = brasero_pref_new ();
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (app));
+
+	/* FIXME! This is a bad idea and needs fixing */
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (priv->mainwin));
 
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
@@ -526,6 +1009,9 @@ on_about_cb (GtkAction *action, BraseroApp *app)
         };
 
 	gchar  *license, *comments;
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
 
 	comments = g_strdup (_("A simple to use CD/DVD burning application for GNOME"));
 
@@ -535,7 +1021,8 @@ on_about_cb (GtkAction *action, BraseroApp *app)
                              _(license_part[2]),
 			     NULL);
 
-	gtk_show_about_dialog (GTK_WINDOW (GTK_WIDGET (app)),
+	/* This can only be shown from the main window so no need for toplevel */
+	gtk_show_about_dialog (GTK_WINDOW (GTK_WIDGET (priv->mainwin)),
 			       "program-name", "Brasero",
 			       "comments", comments,
 			       "version", VERSION,
@@ -562,20 +1049,19 @@ on_about_cb (GtkAction *action, BraseroApp *app)
 	g_free (license);
 }
 
-#ifdef BUILD_GNOME2
-
 static void
 on_help_cb (GtkAction *action, BraseroApp *app)
 {
 	GError *error = NULL;
+	BraseroAppPrivate *priv;
 
- 	gnome_help_display ("brasero.xml",
-			     NULL,
-			     &error);
+	priv = BRASERO_APP_PRIVATE (app);
+
+ 	gtk_show_uri (NULL, "ghelp:brasero", gtk_get_current_event_time (), &error);
    	if (error) {
 		GtkWidget *d;
         
-		d = gtk_message_dialog_new (GTK_WINDOW (app),
+		d = gtk_message_dialog_new (GTK_WINDOW (priv->mainwin),
 					    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 					    GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
 					    "%s", error->message);
@@ -586,16 +1072,14 @@ on_help_cb (GtkAction *action, BraseroApp *app)
 	}
 }
 
-#endif
-
 static gboolean
 on_window_state_changed_cb (GtkWidget *widget,
 			    GdkEventWindowState *event,
-			    gpointer *NULL_data)
+			    BraseroApp *app)
 {
 	BraseroAppPrivate *priv;
 
-	priv = BRASERO_APP_PRIVATE (widget);
+	priv = BRASERO_APP_PRIVATE (app);
 
 	if (event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED)
 		priv->is_maximised = 1;
@@ -608,11 +1092,11 @@ on_window_state_changed_cb (GtkWidget *widget,
 static gboolean
 on_configure_event_cb (GtkWidget *widget,
 		       GdkEventConfigure *event,
-		       gpointer NULL_data)
+		       BraseroApp *app)
 {
 	BraseroAppPrivate *priv;
 
-	priv = BRASERO_APP_PRIVATE (widget);
+	priv = BRASERO_APP_PRIVATE (app);
 
 	if (!priv->is_maximised) {
 		priv->width = event->width;
@@ -808,8 +1292,8 @@ brasero_caps_changed_cb (BraseroPluginManager *manager,
 		gtk_widget_set_sensitive (widget, TRUE);
 }
 
-static void
-brasero_app_init (BraseroApp *object)
+void
+brasero_app_create_mainwin (BraseroApp *app)
 {
 	GtkWidget *hbox;
 	GtkWidget *menubar;
@@ -819,45 +1303,48 @@ brasero_app_init (BraseroApp *object)
 	GtkActionGroup *action_group;
 	BraseroPluginManager *plugin_manager;
 
+	priv = BRASERO_APP_PRIVATE (app);
+
 	/* New window */
-	priv = BRASERO_APP_PRIVATE (object);
-	g_set_application_name (_("Brasero Disc Burner"));
+	priv->mainwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_icon_name (GTK_WINDOW (priv->mainwin), "brasero");
 
-	gtk_window_set_default_icon_name ("brasero");
-	gtk_window_set_icon_name (GTK_WINDOW (object), "brasero");
-
-	g_signal_connect (G_OBJECT (object), "delete-event",
-			  G_CALLBACK (on_delete_cb), object);
-	g_signal_connect (G_OBJECT (object), "destroy",
-			  G_CALLBACK (on_destroy_cb), object);
+	g_signal_connect (G_OBJECT (priv->mainwin),
+			  "delete-event",
+			  G_CALLBACK (on_delete_cb),
+			  app);
+	g_signal_connect (G_OBJECT (priv->mainwin),
+			  "destroy",
+			  G_CALLBACK (on_destroy_cb),
+			  app);
 
 	/* contents */
 	priv->contents = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (priv->contents);
 
-	gtk_container_add (GTK_CONTAINER (object), priv->contents);
+	gtk_container_add (GTK_CONTAINER (priv->mainwin), priv->contents);
 
 	/* menu and toolbar */
 	priv->manager = gtk_ui_manager_new ();
 	g_signal_connect (priv->manager,
 			  "connect-proxy",
 			  G_CALLBACK (brasero_connect_ui_manager_proxy_cb),
-			  object);
+			  app);
 	g_signal_connect (priv->manager,
 			  "disconnect-proxy",
 			  G_CALLBACK (brasero_disconnect_ui_manager_proxy_cb),
-			  object);
+			  app);
 
 	action_group = gtk_action_group_new ("MenuActions");
 	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions (action_group,
 				      entries,
 				      G_N_ELEMENTS (entries),
-				      object);
+				      app);
 
 	gtk_ui_manager_insert_action_group (priv->manager, action_group, 0);
 
-	brasero_app_add_recent (object, action_group);
+	brasero_app_add_recent (app, action_group);
 
 	if (!gtk_ui_manager_add_ui_from_string (priv->manager, description, -1, &error)) {
 		g_message ("building menus failed: %s", error->message);
@@ -906,39 +1393,73 @@ brasero_app_init (BraseroApp *object)
 	g_signal_connect (plugin_manager,
 			  "caps-changed",
 			  G_CALLBACK (brasero_caps_changed_cb),
-			  object);
+			  app);
 
 	/* add accelerators */
 	accel_group = gtk_ui_manager_get_accel_group (priv->manager);
-	gtk_window_add_accel_group (GTK_WINDOW (object), accel_group);
+	gtk_window_add_accel_group (GTK_WINDOW (priv->mainwin), accel_group);
 
 	/* set up the window geometry */
-	gtk_window_set_position (GTK_WINDOW (object), GTK_WIN_POS_CENTER);
+	gtk_window_set_position (GTK_WINDOW (priv->mainwin),
+				 GTK_WIN_POS_CENTER);
 
-#ifdef BUILD_GNOME2
-
-	brasero_session_connect (object);
-
-#endif
-
-	g_signal_connect (object,
+	g_signal_connect (priv->mainwin,
 			  "window-state-event",
 			  G_CALLBACK (on_window_state_changed_cb),
-			  NULL);
-	g_signal_connect (object,
+			  app);
+	g_signal_connect (priv->mainwin,
 			  "configure-event",
 			  G_CALLBACK (on_configure_event_cb),
-			  NULL);
+			  app);
+
+	gtk_widget_realize (GTK_WIDGET (priv->mainwin));
+
+	if (priv->parent) {
+		gtk_window_set_modal (GTK_WINDOW (priv->mainwin), TRUE);
+		gdk_window_set_transient_for (GTK_WIDGET (priv->mainwin)->window, priv->parent);
+	}
+
+	brasero_app_load_window_state (app);
+}
+
+void
+brasero_app_run_mainwin (BraseroApp *app)
+{
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+
+	gtk_widget_show (GTK_WIDGET (priv->mainwin));
+	gtk_main ();
+}
+
+static void
+brasero_app_init (BraseroApp *object)
+{
+	BraseroAppPrivate *priv;
+
+	/* Connect to session */
+	brasero_session_connect (object);
+
+	priv = BRASERO_APP_PRIVATE (object);
+
+	g_set_application_name (_("Brasero Disc Burner"));
+	gtk_window_set_default_icon_name ("brasero");
 }
 
 static void
 brasero_app_finalize (GObject *object)
 {
+	BraseroAppPrivate *priv;
 
-#ifdef BUILD_GNOME2
+	priv = BRASERO_APP_PRIVATE (object);
 
-       brasero_session_disconnect (BRASERO_APP (object));
-#endif
+	brasero_session_disconnect (BRASERO_APP (object));
+
+	if (priv->saved_contents) {
+		g_free (priv->saved_contents);
+		priv->saved_contents = NULL;
+	}
 
 	G_OBJECT_CLASS (brasero_app_parent_class)->finalize (object);
 }
@@ -953,10 +1474,8 @@ brasero_app_class_init (BraseroAppClass *klass)
 	object_class->finalize = brasero_app_finalize;
 }
 
-GtkWidget *
+BraseroApp *
 brasero_app_new (void)
 {
-	return g_object_new (BRASERO_TYPE_APP,
-			     "type", GTK_WINDOW_TOPLEVEL,
-			     NULL);
+	return g_object_new (BRASERO_TYPE_APP, NULL);
 }
