@@ -1109,7 +1109,8 @@ brasero_burn_dialog_action_changed_real (BraseroBurnDialog *dialog,
 					  action,
 					  string);
 	brasero_tray_icon_set_action (BRASERO_TRAYICON (dialog->priv->tray),
-				      action);
+				      action,
+				      string);
 }
 
 static void
@@ -1232,10 +1233,6 @@ brasero_burn_dialog_init (BraseroBurnDialog * obj)
 	gtk_window_set_default_size (GTK_WINDOW (obj), 500, 0);
 
 	gtk_dialog_set_has_separator (GTK_DIALOG (obj), FALSE);
-	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (obj), FALSE);
-	gtk_window_set_skip_pager_hint (GTK_WINDOW (obj), FALSE);
-	gtk_window_set_type_hint (GTK_WINDOW (obj), GDK_WINDOW_TYPE_HINT_NORMAL);
-	gtk_window_set_position (GTK_WINDOW (obj),GTK_WIN_POS_CENTER);
 
 	obj->priv->tray = brasero_tray_icon_new ();
 	g_signal_connect (obj->priv->tray,
@@ -1485,7 +1482,8 @@ brasero_burn_dialog_setup_session (BraseroBurnDialog *dialog,
 					  NULL);
 
 	brasero_tray_icon_set_action (BRASERO_TRAYICON (dialog->priv->tray),
-				      BRASERO_BURN_ACTION_NONE);
+				      BRASERO_BURN_ACTION_NONE,
+				      NULL);
 
 	if (dialog->priv->total_time)
 		g_timer_destroy (dialog->priv->total_time);
@@ -2104,20 +2102,17 @@ brasero_burn_dialog_cancel_dialog (GtkWidget *toplevel)
 	return (result != GTK_RESPONSE_OK);
 }
 
-static gboolean
+gboolean
 brasero_burn_dialog_cancel (BraseroBurnDialog *dialog)
 {
-	if (dialog->priv->burn) {
-		BraseroBurnResult result;
+	if (!dialog->priv->burn)
+		return TRUE;
 
-		result = brasero_burn_cancel (dialog->priv->burn, TRUE);
+	if (brasero_burn_cancel (dialog->priv->burn, TRUE) == BRASERO_BURN_DANGEROUS) {
+		if (!brasero_burn_dialog_cancel_dialog (GTK_WIDGET (dialog)))
+			return FALSE;
 
-		if (result == BRASERO_BURN_DANGEROUS) {
-			if (brasero_burn_dialog_cancel_dialog (GTK_WIDGET (dialog)))
-				brasero_burn_cancel (dialog->priv->burn, FALSE);
-			else
-				return FALSE;
-		}
+		brasero_burn_cancel (dialog->priv->burn, FALSE);
 	}
 
 	return TRUE;
