@@ -31,9 +31,12 @@
 #include <gtk/gtk.h>
 
 #include "brasero-medium-selection.h"
+#include "brasero-utils.h"
+
 #include "burn-medium.h"
 #include "burn-volume-obj.h"
 #include "burn-basics.h"
+#include "burn-medium-monitor.h"
 
 typedef struct _BraseroMediumSelectionPrivate BraseroMediumSelectionPrivate;
 struct _BraseroMediumSelectionPrivate
@@ -45,6 +48,14 @@ struct _BraseroMediumSelectionPrivate
 
 #define BRASERO_MEDIUM_SELECTION_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_MEDIUM_SELECTION, BraseroMediumSelectionPrivate))
 
+typedef enum {
+	ADDED_SIGNAL,
+	REMOVED_SIGNAL,
+	LAST_SIGNAL
+} BraseroMediumSelectionSignalType;
+
+static guint brasero_medium_selection_signals [LAST_SIGNAL] = { 0 };
+
 enum {
 	MEDIUM_COL,
 	NAME_COL,
@@ -54,33 +65,174 @@ enum {
 
 G_DEFINE_TYPE (BraseroMediumSelection, brasero_medium_selection, GTK_TYPE_COMBO_BOX);
 
-enum {
-	MEDIUM_CHANGED,
-	LAST_SIGNAL
-};
-static gulong medium_selection_signals [LAST_SIGNAL];
+void
+brasero_medium_selection_foreach (BraseroMediumSelection *selection,
+				  BraseroMediumSelectionFunc function,
+				  gpointer callback_data)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (selection));
+
+	if (!gtk_tree_model_get_iter_first (model, &iter))
+		return;
+
+	do {
+		BraseroMedium *medium;
+
+		medium = NULL;
+		gtk_tree_model_get (model, &iter,
+				    MEDIUM_COL, &medium,
+				    -1);
+
+		/* The following can happen when there isn't any medium */
+		if (!medium)
+			return;
+
+		if (!function (medium, callback_data))
+			break;
+
+	} while (gtk_tree_model_iter_next (model, &iter));
+}
+
+static gchar *
+brasero_medium_selection_get_medium_string (BraseroMediumSelection *self,
+					    BraseroMedium *medium)
+{
+	gchar *label;
+	gint64 size = 0;
+	gchar *size_string;
+	gchar *medium_name;
+	BraseroMedia media;
+	BraseroMediumSelectionClass *klass;
+
+	klass = BRASERO_MEDIUM_SELECTION_GET_CLASS (self);
+	if (klass->format_medium_string) {
+		gchar *label;
+
+		label = klass->format_medium_string (self, medium);
+		if (label)
+			return label;
+	}
+
+	medium_name = brasero_volume_get_name (BRASERO_VOLUME (medium));
+	if (brasero_medium_get_status (medium) & BRASERO_MEDIUM_FILE)
+		return medium_name;
+
+	media = brasero_medium_get_status (medium);
+	if (media & BRASERO_MEDIUM_BLANK) {
+		/* NOTE for translators, the first %s is the medium name */
+		label = g_strdup_printf (_("%s: empty"), medium_name);
+		g_free (medium_name);
+
+		return label;
+	}
+
+	brasero_medium_get_data_size (medium,
+				      &size,
+				      NULL);
+
+	/* format the size */
+	if (media & BRASERO_MEDIUM_HAS_DATA) {
+		size_string = g_format_size_for_display (size);
+		/* NOTE for translators: the first %s is the medium name, the
+		 * second %s is the space (kio, gio) used by data on the disc.
+		 */
+		label = g_strdup_printf (_("%s: %s"),
+					 medium_name,
+					 size_string);
+	}
+	else {
+		size_string = brasero_utils_get_time_string_from_size (size,
+								       TRUE,
+								       TRUE);
+		/* NOTE for translators: the first %s is the medium name, the
+		 * second %s is the space (time) used by data on the disc.
+		 * I really don't know if I should set this string as
+		 * translatable. */
+		label = g_strdup_printf (_("%s: %s"),
+					 medium_name,
+					 size_string);
+	}
+
+	g_free (medium_name);
+	g_free (size_string);
+
+	return label;
+}
+
+void
+brasero_medium_selection_update_media_string (BraseroMediumSelection *self)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gboolean valid;
+
+	valid = FALSE;
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (self));
+	if (!gtk_tree_model_get_iter_first (model, &iter))
+		return;
+
+	do {
+		BraseroMedium *medium;
+		gchar *label;
+
+		medium = NULL;
+		gtk_tree_model_get (model, &iter,
+				    MEDIUM_COL, &medium,
+				    -1);
+		if (!medium)
+			continue;
+
+		label = brasero_medium_selection_get_medium_string (self, medium);
+		gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+				    NAME_COL, label,
+				    -1);
+
+		g_object_unref (medium);
+		g_free (label);
+	} while (gtk_tree_model_iter_next (model, &iter));
+}
+
+static void
+brasero_drive_selection_set_tooltip (BraseroMediumSelection *self)
+{
+	BraseroMediumSelectionPrivate *priv;
+	BraseroMedium *medium;
+	gchar *tooltip;
+
+	priv = BRASERO_MEDIUM_SELECTION_PRIVATE (self);
+
+	medium = brasero_medium_selection_get_active (self);
+	if (medium) {
+		tooltip = brasero_medium_get_tooltip (medium);
+		g_object_unref (medium);
+	}
+	else
+		tooltip = NULL;
+
+	gtk_widget_set_tooltip_text (GTK_WIDGET (self), tooltip);
+	g_free (tooltip);
+}
 
 static void
 brasero_medium_selection_changed (GtkComboBox *box)
 {
-	GtkTreeIter iter;
-
-	if (gtk_combo_box_get_active_iter (box, &iter))
-		g_signal_emit (box,
-			       medium_selection_signals [MEDIUM_CHANGED],
-			       0);
+	brasero_drive_selection_set_tooltip (BRASERO_MEDIUM_SELECTION (box));
 }
 
-void
+gboolean
 brasero_medium_selection_set_active (BraseroMediumSelection *self,
 				     BraseroMedium *medium)
 {
+	gboolean result = FALSE;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (self));
 	if (!gtk_tree_model_get_iter_first (model, &iter))
-		return;
+		return FALSE;
 
 	do {
 		BraseroMedium *iter_medium;
@@ -90,16 +242,18 @@ brasero_medium_selection_set_active (BraseroMediumSelection *self,
 				    -1);
 
 		if (medium == iter_medium) {
-			g_object_unref (iter_medium);
+			if (iter_medium)
+				g_object_unref (iter_medium);
+
 			gtk_combo_box_set_active_iter (GTK_COMBO_BOX (self), &iter);
-			g_signal_emit (self,
-				       medium_selection_signals [MEDIUM_CHANGED],
-				       0);
+			result = TRUE;
 			break;
 		}
 
 		g_object_unref (iter_medium);
 	} while (gtk_tree_model_iter_next (model, &iter));
+
+	return result;
 }
 
 BraseroMedium *
@@ -119,6 +273,80 @@ brasero_medium_selection_get_active (BraseroMediumSelection *self)
 	return medium;
 }
 
+BraseroDrive *
+brasero_medium_selection_get_active_drive (BraseroMediumSelection *self)
+{
+	BraseroMedium *medium;
+	BraseroDrive *drive;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (self));
+	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (self), &iter))
+		return NULL;
+
+	gtk_tree_model_get (model, &iter,
+			    MEDIUM_COL, &medium,
+			    -1);
+
+	if (!medium)
+		return NULL;
+
+	drive = brasero_medium_get_drive (medium);
+	g_object_unref (medium);
+	g_object_ref (drive);
+
+	return drive;
+}
+
+static void
+brasero_medium_selection_update_no_disc_entry (BraseroMediumSelection *self,
+					       GtkTreeModel *model,
+					       GtkTreeIter *iter)
+{
+	BraseroMediumMonitor *monitor;
+	GIcon *icon;
+
+	monitor = brasero_medium_monitor_get_default ();
+	if (brasero_medium_monitor_is_probing (monitor)) {
+		icon = g_themed_icon_new_with_default_fallbacks ("image-loading");
+		gtk_list_store_set (GTK_LIST_STORE (model), iter,
+				    NAME_COL, _("Searching for available discs"),
+				    ICON_COL, icon,
+				    -1);
+	}
+	else {
+		icon = g_themed_icon_new_with_default_fallbacks ("drive-optical");
+		gtk_list_store_set (GTK_LIST_STORE (model), iter,
+				    NAME_COL, _("No available disc"),
+				    ICON_COL, icon,
+				    -1);
+	}
+
+	g_object_unref (icon);
+	g_object_unref (monitor);
+
+	gtk_widget_set_sensitive (GTK_WIDGET (self), FALSE);
+	gtk_combo_box_set_active_iter (GTK_COMBO_BOX (self), iter);
+}
+
+static void
+brasero_medium_selection_add_no_disc_entry (BraseroMediumSelection *self)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	BraseroMediumSelectionPrivate *priv;
+
+	priv = BRASERO_MEDIUM_SELECTION_PRIVATE (self);
+
+	/* Nothing's available. Say it. Two cases here, either we're
+	 * still probing drives or there isn't actually any available
+	 * medium. */
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (self));
+	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+	brasero_medium_selection_update_no_disc_entry (self, model, &iter);
+}
+
 void
 brasero_medium_selection_show_type (BraseroMediumSelection *self,
 				    BraseroMediaType type)
@@ -160,9 +388,17 @@ brasero_medium_selection_show_type (BraseroMediumSelection *self,
 			g_object_unref (medium);
 
 			if (!node) {
-				if (gtk_list_store_remove (GTK_LIST_STORE (model), &iter))
+				if (gtk_list_store_remove (GTK_LIST_STORE (model), &iter)) {
+					g_signal_emit (self,
+						       brasero_medium_selection_signals [REMOVED_SIGNAL],
+						       0);
 					continue;
+				}
 
+				/* no more iter in the tree  get out */
+				g_signal_emit (self,
+					       brasero_medium_selection_signals [REMOVED_SIGNAL],
+					       0);
 				break;
 			}
 
@@ -175,13 +411,13 @@ brasero_medium_selection_show_type (BraseroMediumSelection *self,
 		/* add remaining media */
 		for (item = list; item; item = item->next) {
 			gchar *medium_name;
+			GIcon *medium_icon;
 			BraseroMedium *medium;
-			const gchar *medium_icon;
 
 			medium = item->data;
 
-			medium_name = brasero_volume_get_display_label (BRASERO_VOLUME (medium), TRUE);
-			medium_icon = brasero_medium_get_icon (medium);
+			medium_name = brasero_medium_selection_get_medium_string (self, medium);
+			medium_icon = brasero_volume_get_icon (BRASERO_VOLUME (medium));
 
 			gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
@@ -190,31 +426,51 @@ brasero_medium_selection_show_type (BraseroMediumSelection *self,
 					    ICON_COL, medium_icon,
 					    -1);
 			g_free (medium_name);
+			g_signal_emit (self,
+				       brasero_medium_selection_signals [ADDED_SIGNAL],
+				       0);
 		}
 		g_slist_foreach (list, (GFunc) g_object_unref, NULL);
 		g_slist_free (list);
 	}
 
 	if (!gtk_tree_model_get_iter_first (model, &iter)) {
-		/* Nothing's available =(. Say it. */
-		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-		gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-				    NAME_COL, _("There is no available medium. Please insert one."),
-				    -1),
-
-		gtk_combo_box_set_active_iter (GTK_COMBO_BOX (self), &iter);
-		g_signal_emit (self,
-			       medium_selection_signals [MEDIUM_CHANGED],
-			       0);
+		brasero_medium_selection_add_no_disc_entry (self);
 		return;
 	}
 
+	gtk_widget_set_sensitive (GTK_WIDGET (self), TRUE);
 	if (gtk_combo_box_get_active (GTK_COMBO_BOX (self)) == -1)
 		gtk_combo_box_set_active_iter (GTK_COMBO_BOX (self), &iter);
-	else
-		g_signal_emit (self,
-			       medium_selection_signals [MEDIUM_CHANGED],
-			       0);
+}
+
+guint
+brasero_medium_selection_get_drive_num (BraseroMediumSelection *self)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	int num = 0;
+
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (self));
+	if (!gtk_tree_model_get_iter_first (model, &iter))
+		return 0;
+
+	do {
+		BraseroMedium *medium;
+
+		medium = NULL;
+		gtk_tree_model_get (model, &iter,
+				    MEDIUM_COL, &medium,
+				    -1);
+		if (!medium)
+			continue;
+
+		g_object_unref (medium);
+		num ++;
+
+	} while (gtk_tree_model_iter_next (model, &iter));
+
+	return num;
 }
 
 static void
@@ -223,11 +479,11 @@ brasero_medium_selection_medium_added_cb (BraseroMediumMonitor *monitor,
 					  BraseroMediumSelection *self)
 {
 	BraseroMediumSelectionPrivate *priv;
-	const gchar *medium_icon;
 	gboolean add = FALSE;
 	GtkTreeModel *model;
 	BraseroDrive *drive;
 	gchar *medium_name;
+	GIcon *medium_icon;
 	GtkTreeIter iter;
 
 	priv = BRASERO_MEDIUM_SELECTION_PRIVATE (self);
@@ -251,11 +507,27 @@ brasero_medium_selection_medium_added_cb (BraseroMediumMonitor *monitor,
 			add = TRUE;
 	}
 
-	if (!add)
-		return;
-
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (self));
 
+	if (!add) {
+		/* Try to get the first iter (it shouldn't fail) */
+		if (!gtk_tree_model_get_iter_first (model, &iter)) {
+			brasero_medium_selection_add_no_disc_entry (self);
+			return;
+		}
+
+		/* See if that's a real medium or not; if so, return. */
+		medium = NULL;
+		gtk_tree_model_get (model, &iter,
+				    MEDIUM_COL, &medium,
+				    -1);
+		if (medium)
+			return;
+
+		brasero_medium_selection_update_no_disc_entry (self, model, &iter);
+		return;
+	}
+
 	/* remove warning message */
 	if (gtk_tree_model_get_iter_first (model, &iter)) {
 		BraseroMedium *tmp;
@@ -269,8 +541,8 @@ brasero_medium_selection_medium_added_cb (BraseroMediumMonitor *monitor,
 			g_object_unref (tmp);
 	}
 
-	medium_name = brasero_volume_get_display_label (BRASERO_VOLUME (medium), TRUE);
-	medium_icon = brasero_medium_get_icon (medium);
+	medium_name = brasero_medium_selection_get_medium_string (self, medium);
+	medium_icon = brasero_volume_get_icon (BRASERO_VOLUME (medium));
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 			    MEDIUM_COL, medium,
@@ -279,12 +551,13 @@ brasero_medium_selection_medium_added_cb (BraseroMediumMonitor *monitor,
 			    -1);
 	g_free (medium_name);
 
-	if (gtk_combo_box_get_active (GTK_COMBO_BOX (self)) == -1) {
+	gtk_widget_set_sensitive (GTK_WIDGET (self), TRUE);
+	if (gtk_combo_box_get_active (GTK_COMBO_BOX (self)) == -1)
 		gtk_combo_box_set_active_iter (GTK_COMBO_BOX (self), &iter);
-		g_signal_emit (self,
-			       medium_selection_signals [MEDIUM_CHANGED],
-			       0);
-	}
+
+	g_signal_emit (self,
+		       brasero_medium_selection_signals [ADDED_SIGNAL],
+		       0);
 }
 
 static void
@@ -309,32 +582,25 @@ brasero_medium_selection_medium_removed_cb (BraseroMediumMonitor *monitor,
 		if (medium == iter_medium) {
 			g_object_unref (iter_medium);
 			gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+			g_signal_emit (self,
+				       brasero_medium_selection_signals [REMOVED_SIGNAL],
+				       0);
 			break;
 		}
 
-		g_object_unref (iter_medium);
+		/* Could be NULL if a message "there is no medium ..." is on */
+		if (iter_medium)
+			g_object_unref (iter_medium);
+
 	} while (gtk_tree_model_iter_next (model, &iter));
 
 	if (!gtk_tree_model_get_iter_first (model, &iter)) {
-		/* Nothing's available any more =(. Say it. */
-		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-		gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-				    NAME_COL, _("There is no available medium. Please insert one."),
-				    -1),
-
-		gtk_combo_box_set_active_iter (GTK_COMBO_BOX (self), &iter);
-		g_signal_emit (self,
-			       medium_selection_signals [MEDIUM_CHANGED],
-			       0);
+		brasero_medium_selection_add_no_disc_entry (self);
 		return;
 	}
 
-	if (gtk_combo_box_get_active (GTK_COMBO_BOX (self)) == -1) {
+	if (gtk_combo_box_get_active (GTK_COMBO_BOX (self)) == -1)
 		gtk_combo_box_set_active_iter (GTK_COMBO_BOX (self), &iter);
-		g_signal_emit (self,
-			       medium_selection_signals [MEDIUM_CHANGED],
-			       0);
-	}
 }
 
 static void
@@ -363,18 +629,18 @@ brasero_medium_selection_init (BraseroMediumSelection *object)
 	model = gtk_list_store_new (NUM_COL,
 				    G_TYPE_OBJECT,
 				    G_TYPE_STRING,
-				    G_TYPE_STRING);
+				    G_TYPE_ICON);
 
 	gtk_combo_box_set_model (GTK_COMBO_BOX (object), GTK_TREE_MODEL (model));
 	g_object_unref (model);
 
-/*	renderer = gtk_cell_renderer_pixbuf_new ();
+	renderer = gtk_cell_renderer_pixbuf_new ();
 	g_object_set (renderer, "follow-state", TRUE, NULL);
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (object), renderer, FALSE);
 	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (object), renderer,
-					"icon-name", ICON_COL,
+					"gicon", ICON_COL,
 					NULL);
-*/
+
 	renderer = gtk_cell_renderer_text_new ();
 	g_object_set (renderer, "xpad", 8, NULL);
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (object), renderer, TRUE);
@@ -415,16 +681,26 @@ brasero_medium_selection_class_init (BraseroMediumSelectionClass *klass)
 
 	combo_class->changed = brasero_medium_selection_changed;
 
-	medium_selection_signals [MEDIUM_CHANGED] =
-	    g_signal_new ("medium_changed",
-			  G_TYPE_FROM_CLASS (klass),
-			  G_SIGNAL_RUN_FIRST,
+	brasero_medium_selection_signals [ADDED_SIGNAL] =
+	    g_signal_new ("medium_added",
+			  BRASERO_TYPE_MEDIUM_SELECTION,
+			  G_SIGNAL_RUN_FIRST|G_SIGNAL_ACTION|G_SIGNAL_NO_RECURSE,
 			  0,
-			  NULL, NULL,
+			  NULL,
+			  NULL,
 			  g_cclosure_marshal_VOID__VOID,
 			  G_TYPE_NONE,
+			  0);
+	brasero_medium_selection_signals [REMOVED_SIGNAL] =
+	    g_signal_new ("medium_removed",
+			  BRASERO_TYPE_MEDIUM_SELECTION,
+			  G_SIGNAL_RUN_FIRST|G_SIGNAL_ACTION|G_SIGNAL_NO_RECURSE,
 			  0,
-			  G_TYPE_NONE);
+			  NULL,
+			  NULL,
+			  g_cclosure_marshal_VOID__VOID,
+			  G_TYPE_NONE,
+			  0);
 }
 
 GtkWidget *
