@@ -3,16 +3,16 @@
  *
  *  Mon Nov 27 17:31:40 2006
  *  Copyright  2006  Rouquier Philippe
- *  <Rouquier Philippe@localhost.localdomain>
+ *  <bonfire-app@wanadoo.fr>
  ****************************************************************************/
 
 /*
- * This program is free software; you can redistribute it and/or modify
+ * Brasero is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
@@ -26,12 +26,12 @@
 
 #include "burn-iso-field.h"
 
-gint32
+guint32
 brasero_iso9660_get_733_val (guchar *buffer)
 {
-	gint32 *ptr;
+	guint32 *ptr;
 
-	ptr = (gint32*) buffer;
+	ptr = (guint32*) buffer;
 
-	return GINT32_FROM_LE (*ptr);
+	return GUINT32_FROM_LE (*ptr);
 }
