@@ -307,10 +307,12 @@ brasero_job_allow_deactivation (BraseroJob *self,
 	return BRASERO_BURN_NOT_RUNNING;
 
 error:
+
 	g_set_error (error,
 		     BRASERO_BURN_ERR,
-		     BRASERO_BURN_ERROR_GENERAL,
-		     _("plugin %s did not want to work. Try to deactivate it"),
+		     BRASERO_BURN_ERROR_PLUGIN_MISBEHAVIOR,
+		     /* Translators: %s is the plugin name */
+		     _("\"%s\" did not behave properly"),
 		     G_OBJECT_TYPE_NAME (self));
 	return BRASERO_BURN_ERR;
 }
@@ -402,13 +404,13 @@ brasero_job_check_output_disc_space (BraseroJob *self,
 	/* this is not really an error, we'll probably ask the 
 	 * user to load a new disc */
 	if (output_blocks > media_blocks) {
-		BRASERO_BURN_LOG ("Insufficient space on media %lli/%lli",
+		BRASERO_BURN_LOG ("Insufficient space on disc %lli/%lli",
 				  media_blocks,
 				  output_blocks);
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_MEDIA_SPACE,
-			     _("Insufficient space on media (%lli available for %lli)"),
+			     BRASERO_BURN_ERROR_MEDIUM_SPACE,
+			     _("Not enough space available on the disc (%lli available for %lli)"),
 			     media_blocks,
 			     output_blocks);
 		return BRASERO_BURN_NEED_RELOAD;
@@ -421,9 +423,9 @@ static BraseroBurnResult
 brasero_job_check_output_volume_space (BraseroJob *self,
 				       GError **error)
 {
-	GFile *file;
 	GFileInfo *info;
 	gchar *directory;
+	GFile *file = NULL;
 	struct rlimit limit;
 	guint64 vol_size = 0;
 	gint64 output_size = 0;
@@ -435,7 +437,7 @@ brasero_job_check_output_volume_space (BraseroJob *self,
 
 	priv = BRASERO_JOB_PRIVATE (self);
 
-	/* get the size and filesystem type for the volume first.
+	/* Get the size and filesystem type for the volume first.
 	 * NOTE: since any plugin must output anything LOCALLY, we can then use
 	 * all libc API. */
 	if (!priv->output)
@@ -448,6 +450,29 @@ brasero_job_check_output_volume_space (BraseroJob *self,
 	if (file == NULL)
 		goto error;
 
+	info = g_file_query_info (file,
+				  G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
+				  G_FILE_QUERY_INFO_NONE,
+				  NULL,
+				  error);
+	if (!info)
+		goto error;
+
+	/* Check permissions first */
+	if (!g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE)) {
+		BRASERO_JOB_LOG (self, "No permissions");
+
+		g_object_unref (info);
+		g_object_unref (file);
+		g_set_error (error,
+			     BRASERO_BURN_ERROR,
+			     BRASERO_BURN_ERROR_PERMISSION,
+			     _("You do not have the required permission to write at this location"));
+		return BRASERO_BURN_ERR;
+	}
+	g_object_unref (info);
+
+	/* Now size left */
 	info = g_file_query_filesystem_info (file,
 					     G_FILE_ATTRIBUTE_FILESYSTEM_FREE ","
 					     G_FILE_ATTRIBUTE_FILESYSTEM_TYPE,
@@ -457,8 +482,7 @@ brasero_job_check_output_volume_space (BraseroJob *self,
 		goto error;
 
 	g_object_unref (file);
-
-	brasero_job_get_session_output_size (BRASERO_JOB (self), NULL, &output_size);
+	file = NULL;
 
 	/* Now check the filesystem type: the problem here is that some
 	 * filesystems have a maximum file size limit of 4 GiB and more than
@@ -469,10 +493,11 @@ brasero_job_check_output_volume_space (BraseroJob *self,
 	if (output_size >= 2147483648ULL
 	&&  filesystem
 	&& !strcmp (filesystem, "msdos")) {
+		g_object_unref (info);
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_DISK_SPACE,
-			     _("The filesystem you chose to store the temporary image on cannot hold files with a size over 2 GiB."));
+			     _("The filesystem you chose to store the temporary image on cannot hold files with a size over 2 GiB"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -498,7 +523,8 @@ brasero_job_check_output_volume_space (BraseroJob *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_DISK_SPACE,
-			     strerror (errno));
+			     "%s",
+			     g_strerror (errno));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -508,7 +534,7 @@ brasero_job_check_output_volume_space (BraseroJob *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_DISK_SPACE,
-			     _("the selected location does not have enough free space to store the disc image (%ld MiB needed)"),
+			     _("The location you chose to store the temporary image on does not have enough free space for the disc image (%ld MiB needed)"),
 			     (unsigned long) output_size / 1048576);
 		return BRASERO_BURN_ERR;
 	}
@@ -521,8 +547,11 @@ error:
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the size of the volume can't be checked (Unknown error)"));
-	g_object_unref (file);
+			     _("The size of the volume could not be retrieved"));
+
+	if (file)
+		g_object_unref (file);
+
 	return BRASERO_BURN_ERR;
 }
 
@@ -534,6 +563,7 @@ brasero_job_set_output_file (BraseroJob *self,
 	BraseroBurnResult result;
 	BraseroJobPrivate *priv;
 	BraseroBurnFlag flags;
+	gint64 output_size = 0;
 	gchar *image = NULL;
 	gchar *toc = NULL;
 
@@ -541,29 +571,99 @@ brasero_job_set_output_file (BraseroJob *self,
 
 	/* no next job so we need a file pad */
 	session = brasero_task_ctx_get_session (priv->ctx);
+
+	/* If the plugin is not supposed to output anything, then don't test */
+	brasero_job_get_session_output_size (BRASERO_JOB (self), NULL, &output_size);
+
+	/* This should be re-enabled when we make sure all plugins (like vcd)
+	 * don't advertize an output of 0 whereas it's not true. Maybe we could
+	 * use -1 for plugins that don't output. */
+	/* if (!output_size)
+		return BRASERO_BURN_OK; */
+
+	flags = brasero_burn_session_get_flags (session);
 	if (priv->type.type == BRASERO_TRACK_TYPE_IMAGE) {
 		BraseroImageFormat format;
 
 		/* check if that's the last task */
 		format = brasero_burn_session_get_output_format (session);
-		if (priv->type.subtype.img_format == format)
+		if (priv->type.subtype.img_format == format) {
+			BraseroTrackType input = { 0, };
+
 			result = brasero_burn_session_get_output (session,
 								  &image,
 								  &toc,
 								  error);
-		else
+
+			/* check paths are set */
+			if (!image
+			|| (priv->type.subtype.img_format != BRASERO_IMAGE_FORMAT_BIN && !toc)) {
+				g_set_error (error,
+					     BRASERO_BURN_ERROR,
+					     BRASERO_BURN_ERROR_OUTPUT_NONE,
+					     _("No path was specified for the image output"));
+				return BRASERO_BURN_ERR;
+			}
+
+			brasero_burn_session_get_input_type (session,
+							     &input);
+
+			/* if input is the same as output, then that's a
+			 * processing task and there's no need to check if the
+			 * output already exists since it will (but that's OK) */
+			if (input.type == BRASERO_TRACK_TYPE_IMAGE
+			&&  input.subtype.img_format == priv->type.subtype.img_format) {
+				BRASERO_BURN_LOG ("Processing task, skipping check size");
+				priv->output = g_new0 (BraseroJobOutput, 1);
+				priv->output->image = image;
+				priv->output->toc = toc;
+				return BRASERO_BURN_OK;
+			}
+
+			/* There must be an image at least */
+			if ((flags & BRASERO_BURN_FLAG_DONT_OVERWRITE)
+			&&   g_file_test (image, G_FILE_TEST_EXISTS)) {
+				BRASERO_BURN_LOG ("Problem with image existence");
+				g_set_error (error,
+					     BRASERO_BURN_ERROR,
+					     BRASERO_BURN_ERROR_GENERAL,
+					     _("\"%s\" already exists"),
+					     image);
+				g_free (toc);
+				g_free (image);
+				return BRASERO_BURN_ERR;
+			}
+
+			if (priv->type.subtype.img_format != BRASERO_IMAGE_FORMAT_BIN) {
+				/* There must a toc file in this case */
+				if ((flags & BRASERO_BURN_FLAG_DONT_OVERWRITE)
+				&&   g_file_test (toc, G_FILE_TEST_EXISTS)) {
+					BRASERO_BURN_LOG ("Problem with toc existence");
+					g_set_error (error,
+						     BRASERO_BURN_ERROR,
+						     BRASERO_BURN_ERROR_GENERAL,
+						     _("\"%s\" already exists"),
+						     toc);
+					g_free (toc);
+					g_free (image);
+					return BRASERO_BURN_ERR;
+				}
+			}
+		}
+		else {
+			/* NOTE: no need to check for the existence here */
 			result = brasero_burn_session_get_tmp_image (session,
 								     priv->type.subtype.img_format,
 								     &image,
 								     &toc,
 								     error);
-
-		if (result != BRASERO_BURN_OK)
-			return result;
+			if (result != BRASERO_BURN_OK)
+				return result;
+		}
 
 		BRASERO_JOB_LOG (self, "output set (IMAGE) image = %s toc = %s",
 				 image,
-		    toc ? toc : "nil");
+				 toc ? toc : "none");
 	}
 	else if (priv->type.type == BRASERO_TRACK_TYPE_AUDIO) {
 		/* NOTE: this one can only a temporary file */
@@ -571,8 +671,7 @@ brasero_job_set_output_file (BraseroJob *self,
 							    ".cdr",
 							    &image,
 							    error);
-		BRASERO_JOB_LOG (self, "Output set (AUDIO) image = %s",
-				 image);
+		BRASERO_JOB_LOG (self, "Output set (AUDIO) image = %s", image);
 	}
 	else /* other types don't need an output */
 		return BRASERO_BURN_OK;
@@ -584,8 +683,6 @@ brasero_job_set_output_file (BraseroJob *self,
 	priv->output->image = image;
 	priv->output->toc = toc;
 
-	flags = brasero_burn_session_get_flags (session);
-
 	if (flags & BRASERO_BURN_FLAG_CHECK_SIZE)
 		return brasero_job_check_output_volume_space (self, error);
 
@@ -669,11 +766,14 @@ brasero_job_item_start (BraseroTaskItem *item,
 
 		/* setup a pipe */
 		if (pipe (fd)) {
+                        int errsv = errno;
+
+			BRASERO_BURN_LOG ("A pipe couldn't be created");
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
-				     _("the pipe couldn't be created (%s)"),
-				     strerror (errno));
+				     _("An internal error occured (%s)"),
+				     g_strerror (errsv));
 
 			return BRASERO_BURN_ERR;
 		}
@@ -700,9 +800,8 @@ brasero_job_item_start (BraseroTaskItem *item,
 		if (priv->linked) {
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
-				     _("plugin %s did not work properly.\n"
-				       "Try to deactivate it"),
+				     BRASERO_BURN_ERROR_PLUGIN_MISBEHAVIOR,
+				     _("\"%s\" did not behave properly"),
 				     G_OBJECT_TYPE_NAME (self));
 			return BRASERO_BURN_ERR;
 		}
@@ -714,9 +813,8 @@ brasero_job_item_start (BraseroTaskItem *item,
 		if (action != BRASERO_JOB_ACTION_SIZE) {
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
-				     _("plugin %s didn't work properly (it does not support operation).\n"
-				       "Try to deactivate it"),
+				     BRASERO_BURN_ERROR_PLUGIN_MISBEHAVIOR,
+				     _("\"%s\" did not behave properly"),
 				     G_OBJECT_TYPE_NAME (self));
 			return BRASERO_BURN_ERR;
 		}
@@ -934,8 +1032,9 @@ brasero_job_finished_session (BraseroJob *self)
 		 * only call for a stop on an error. */
 		BRASERO_JOB_LOG (self, "is not a leader");
 		error = g_error_new (BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
-				     _("a plugin did not behave properly"));
+				     BRASERO_BURN_ERROR_PLUGIN_MISBEHAVIOR,
+				     _("\"%s\" did not behave properly"),
+				     G_OBJECT_TYPE_NAME (self));
 		return brasero_task_ctx_error (priv->ctx,
 					       BRASERO_BURN_ERR,
 					       error);
@@ -976,7 +1075,7 @@ brasero_job_finished_track (BraseroJob *self)
 
 	priv = BRASERO_JOB_PRIVATE (self);
 
-	BRASERO_JOB_LOG (self, "finished track successfully");
+	BRASERO_JOB_LOG (self, "Finished track successfully");
 
 	/* we first check if it's the first job */
 	if (brasero_job_is_first_active (self)) {
@@ -1023,8 +1122,9 @@ brasero_job_finished_track (BraseroJob *self)
 		 * for a stop on an error. */
 		BRASERO_JOB_LOG (self, "is not a leader");
 		error = g_error_new (BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
-				     _("a plugin did not behave properly"));
+				     BRASERO_BURN_ERROR_PLUGIN_MISBEHAVIOR,
+				     _("\"%s\" did not behave properly"),
+				     G_OBJECT_TYPE_NAME (self));
 		return brasero_task_ctx_error (priv->ctx, BRASERO_BURN_ERR, error);
 	}
 
@@ -1119,18 +1219,20 @@ brasero_job_set_nonblocking_fd (int fd, GError **error)
 		 * automatically but still offer that possibility. */
 		flags |= O_NONBLOCK;
 		if (fcntl (fd, F_SETFL, flags) == -1) {
+			BRASERO_BURN_LOG ("couldn't set non blocking mode");
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
-				     _("couldn't set non blocking mode"));
+				     _("An internal error occured"));
 			return BRASERO_BURN_ERR;
 		}
 	}
 	else {
+		BRASERO_BURN_LOG ("couldn't get pipe flags");
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("couldn't get pipe flags"));
+			     _("An internal error occured"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -1510,9 +1612,11 @@ brasero_job_get_speed (BraseroJob *self, guint *speed)
 	rate = brasero_burn_session_get_rate (session);
 
 	media = brasero_burn_session_get_dest_media (session);
-	if (media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))
+	if (media & BRASERO_MEDIUM_DVD)
 		*speed = BRASERO_RATE_TO_SPEED_DVD (rate);
-	else 
+	else if (media & BRASERO_MEDIUM_BD)
+		*speed = BRASERO_RATE_TO_SPEED_BD (rate);
+	else
 		*speed = BRASERO_RATE_TO_SPEED_CD (rate);
 
 	return BRASERO_BURN_OK;
@@ -1562,9 +1666,11 @@ brasero_job_get_max_speed (BraseroJob *self, guint *speed)
 	medium = brasero_drive_get_medium (drive);
 	rate = brasero_medium_get_max_write_speed (medium);
 	media = brasero_medium_get_status (medium);
-	if (media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))
+	if (media & BRASERO_MEDIUM_DVD)
 		*speed = BRASERO_RATE_TO_SPEED_DVD (rate);
-	else 
+	else if (media & BRASERO_MEDIUM_BD)
+		*speed = BRASERO_RATE_TO_SPEED_BD (rate);
+	else
 		*speed = BRASERO_RATE_TO_SPEED_CD (rate);
 
 	return BRASERO_BURN_OK;
@@ -1581,12 +1687,10 @@ brasero_job_get_tmp_file (BraseroJob *self,
 
 	priv = BRASERO_JOB_PRIVATE (self);
 	session = brasero_task_ctx_get_session (priv->ctx);
-	brasero_burn_session_get_tmp_file (session,
-					   suffix,
-					   output,
-					   error);
-
-	return BRASERO_BURN_OK;
+	return brasero_burn_session_get_tmp_file (session,
+						  suffix,
+						  output,
+						  error);
 }
 
 BraseroBurnResult
@@ -1665,7 +1769,8 @@ brasero_job_start_progress (BraseroJob *self,
 {
 	BraseroJobPrivate *priv;
 
-	BRASERO_JOB_DEBUG (self);
+	/* Turn this off as otherwise it floods bug reports */
+	// BRASERO_JOB_DEBUG (self);
 
 	priv = BRASERO_JOB_PRIVATE (self);
 	if (priv->next)
@@ -1678,29 +1783,15 @@ brasero_job_start_progress (BraseroJob *self,
  * these should be used to set the different values of the task by the jobs
  */
 
-BraseroBurnResult
-brasero_job_add_wrong_checksum (BraseroJob *self,
-				const gchar *path)
-{
-	BraseroJobPrivate *priv;
-	BraseroBurnSession *session;
-
-	BRASERO_JOB_DEBUG (self);
-
-	priv = BRASERO_JOB_PRIVATE (self);
-	session = brasero_task_ctx_get_session (priv->ctx);
-
-	brasero_burn_session_add_wrong_checksum (session, path);
-	return BRASERO_BURN_OK;
-}
-
 BraseroBurnResult
 brasero_job_set_progress (BraseroJob *self,
 			  gdouble progress)
 {
 	BraseroJobPrivate *priv;
 
-	BRASERO_JOB_LOG (self, "Called brasero_job_set_progress (%lf)", progress);
+	/* Turn this off as it floods bug reports */
+	//BRASERO_JOB_LOG (self, "Called brasero_job_set_progress (%lf)", progress);
+
 	priv = BRASERO_JOB_PRIVATE (self);
 	if (priv->next)
 		return BRASERO_BURN_ERR;
@@ -1761,7 +1852,8 @@ brasero_job_set_rate (BraseroJob *self,
 {
 	BraseroJobPrivate *priv;
 
-	BRASERO_JOB_DEBUG (self);
+	/* Turn this off as otherwise it floods bug reports */
+	// BRASERO_JOB_DEBUG (self);
 
 	priv = BRASERO_JOB_PRIVATE (self);
 	if (priv->next)
@@ -1802,7 +1894,8 @@ brasero_job_set_written_track (BraseroJob *self,
 {
 	BraseroJobPrivate *priv;
 
-	BRASERO_JOB_DEBUG (self);
+	/* Turn this off as otherwise it floods bug reports */
+	// BRASERO_JOB_DEBUG (self);
 
 	priv = BRASERO_JOB_PRIVATE (self);
 	if (priv->next)
@@ -1817,7 +1910,8 @@ brasero_job_set_written_session (BraseroJob *self,
 {
 	BraseroJobPrivate *priv;
 
-	BRASERO_JOB_DEBUG (self);
+	/* Turn this off as otherwise it floods bug reports */
+	// BRASERO_JOB_DEBUG (self);
 
 	priv = BRASERO_JOB_PRIVATE (self);
 	if (priv->next)
