@@ -102,60 +102,6 @@ brasero_volume_get_gvolume (BraseroVolume *self)
 	return volume;
 }
 
-static GDrive *
-brasero_volume_get_gdrive (BraseroVolume *self)
-{
-	const gchar *volume_path = NULL;
-	BraseroVolumePrivate *priv;
-	GVolumeMonitor *monitor;
-	GDrive *gdrive = NULL;
-	BraseroDrive *drive;
-	GList *drives;
-	GList *iter;
-
-	priv = BRASERO_VOLUME_PRIVATE (self);
-
-	drive = brasero_medium_get_drive (BRASERO_MEDIUM (self));
-
-#if defined(HAVE_STRUCT_USCSI_CMD)
-	volume_path = brasero_drive_get_block_device (drive);
-#else
-	volume_path = brasero_drive_get_device (drive);
-#endif
-
-	/* NOTE: medium-monitor already holds a reference for GVolumeMonitor */
-	monitor = g_volume_monitor_get ();
-	drives = g_volume_monitor_get_connected_drives (monitor);
-	g_object_unref (monitor);
-
-	for (iter = drives; iter; iter = iter->next) {
-		gchar *device_path;
-		GDrive *tmp;
-
-		tmp = iter->data;
-		device_path = g_drive_get_identifier (tmp, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
-		if (!device_path)
-			continue;
-
-		BRASERO_BURN_LOG ("Found drive %s", device_path);
-		if (!strcmp (device_path, volume_path)) {
-			gdrive = tmp;
-			g_free (device_path);
-			g_object_ref (gdrive);
-			break;
-		}
-
-		g_free (device_path);
-	}
-	g_list_foreach (drives, (GFunc) g_object_unref, NULL);
-	g_list_free (drives);
-
-	if (!drive)
-		BRASERO_BURN_LOG ("No drive found for medium");
-
-	return gdrive;
-}
-
 gboolean
 brasero_volume_is_mounted (BraseroVolume *self)
 {
@@ -195,6 +141,9 @@ brasero_volume_is_mounted (BraseroVolume *self)
 
 		mount = iter->data;
 		volume = g_mount_get_volume (mount);
+		if (!volume)
+			continue;
+
 		device_path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 		if (!device_path)
 			continue;		
@@ -637,6 +586,7 @@ brasero_volume_eject (BraseroVolume *self,
 	GDrive *gdrive;
 	GVolume *volume;
 	gboolean result;
+	BraseroDrive *drive;
 	BraseroVolumePrivate *priv;
 
 	BRASERO_BURN_LOG ("Ejecting");
@@ -646,7 +596,8 @@ brasero_volume_eject (BraseroVolume *self,
 
 	priv = BRASERO_VOLUME_PRIVATE (self);
 
-	gdrive = brasero_volume_get_gdrive (self);
+	drive = brasero_medium_get_drive (BRASERO_MEDIUM (self));
+	gdrive = brasero_drive_get_gdrive (drive);
 	if (!gdrive) {
 		BRASERO_BURN_LOG ("No GDrive");
 		goto last_resort;
@@ -720,6 +671,36 @@ brasero_volume_cancel_current_operation (BraseroVolume *self)
 		g_main_loop_quit (priv->loop);
 }
 
+GIcon *
+brasero_volume_get_icon (BraseroVolume *self)
+{
+	GVolume *volume;
+	GMount *mount;
+	GIcon *icon;
+
+	if (!self)
+		return g_themed_icon_new_with_default_fallbacks ("drive-optical");
+
+	if (brasero_medium_get_status (BRASERO_MEDIUM (self)) == BRASERO_MEDIUM_FILE)
+		return g_themed_icon_new_with_default_fallbacks ("iso-image-new");
+
+	volume = brasero_volume_get_gvolume (self);
+	if (!volume)
+		return g_themed_icon_new_with_default_fallbacks ("drive-optical");
+
+	mount = g_volume_get_mount (volume);
+	if (mount) {
+		icon = g_mount_get_icon (mount);
+		g_object_unref (mount);
+	}
+	else
+		icon = g_volume_get_icon (volume);
+
+	g_object_unref (volume);
+
+	return icon;
+}
+
 gchar *
 brasero_volume_get_name (BraseroVolume *self)
 {
