@@ -66,6 +66,8 @@ struct _BraseroChecksumFilesPrivate {
 
 	/* this is for the thread and the end of it */
 	GThread *thread;
+	GMutex *mutex;
+	GCond *cond;
 	gint end_id;
 
 	guint cancel;
@@ -1111,7 +1113,12 @@ brasero_checksum_files_thread (gpointer data)
 						brasero_checksum_files_destroy);
 	}
 
+	/* End thread */
+	g_mutex_lock (priv->mutex);
 	priv->thread = NULL;
+	g_cond_signal (priv->cond);
+	g_mutex_unlock (priv->mutex);
+
 	g_thread_exit (NULL);
 	return NULL;
 }
@@ -1132,10 +1139,12 @@ brasero_checksum_files_start (BraseroJob *job,
 
 	/* we start a thread for the exploration of the graft points */
 	priv = BRASERO_CHECKSUM_FILES_PRIVATE (job);
+	g_mutex_lock (priv->mutex);
 	priv->thread = g_thread_create (brasero_checksum_files_thread,
 					BRASERO_CHECKSUM_FILES (job),
 					TRUE,
 					error);
+	g_mutex_unlock (priv->mutex);
 
 	if (!priv->thread)
 		return BRASERO_BURN_ERR;
@@ -1198,12 +1207,14 @@ brasero_checksum_files_stop (BraseroJob *job,
 
 	priv = BRASERO_CHECKSUM_FILES_PRIVATE (job);
 
+	g_mutex_lock (priv->mutex);
 	if (priv->thread) {
 		priv->cancel = 1;
-		g_thread_join (priv->thread);
+		g_cond_wait (priv->cond, priv->mutex);
 		priv->cancel = 0;
 		priv->thread = NULL;
 	}
+	g_mutex_unlock (priv->mutex);
 
 	if (priv->end_id) {
 		g_source_remove (priv->end_id);
@@ -1225,7 +1236,13 @@ brasero_checksum_files_stop (BraseroJob *job,
 
 static void
 brasero_checksum_files_init (BraseroChecksumFiles *obj)
-{ }
+{	BraseroChecksumFilesPrivate *priv;
+
+	priv = BRASERO_CHECKSUM_FILES_PRIVATE (obj);
+
+	priv->mutex = g_mutex_new ();
+	priv->cond = g_cond_new ();
+}
 
 static void
 brasero_checksum_files_finalize (GObject *object)
@@ -1234,12 +1251,14 @@ brasero_checksum_files_finalize (GObject *object)
 	
 	priv = BRASERO_CHECKSUM_FILES_PRIVATE (object);
 
+	g_mutex_lock (priv->mutex);
 	if (priv->thread) {
 		priv->cancel = 1;
-		g_thread_join (priv->thread);
+		g_cond_wait (priv->cond, priv->mutex);
 		priv->cancel = 0;
 		priv->thread = NULL;
 	}
+	g_mutex_unlock (priv->mutex);
 
 	if (priv->end_id) {
 		g_source_remove (priv->end_id);
@@ -1251,6 +1270,16 @@ brasero_checksum_files_finalize (GObject *object)
 		priv->file = NULL;
 	}
 
+	if (priv->mutex) {
+		g_mutex_free (priv->mutex);
+		priv->mutex = NULL;
+	}
+
+	if (priv->cond) {
+		g_cond_free (priv->cond);
+		priv->cond = NULL;
+	}
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -1278,7 +1307,10 @@ brasero_checksum_files_export_caps (BraseroPlugin *plugin, gchar **error)
 	BraseroPluginConfOption *checksum_type;
 
 	brasero_plugin_define (plugin,
-			       "File checksum",
+			       /* Translators: this is the name of the plugin
+				* which will be translated only when it needs
+				* displaying. */
+			       N_("File Checksum"),
 			       _("Allows to check file integrities on a disc"),
 			       "Philippe Rouquier",
 			       0);
